module.exports = {
  background: require('./property-optimizers/background').level1.property,
  boxShadow: require('./property-optimizers/box-shadow').level1.property,
  borderRadius: require('./property-optimizers/border-radius').level1.property,
  filter: require('./property-optimizers/filter').level1.property,
  fontWeight: require('./property-optimizers/font-weight').level1.property,
  margin: require('./property-optimizers/margin').level1.property,
  outline: require('./property-optimizers/outline').level1.property,
  padding: require('./property-optimizers/padding').level1.property
};
