@font-face {
  src: url("../font/myfont.eot"); /*! IE9 */
  src: url("../font/myfont.eot?#iefix") format("embedded-opentype"), /*! IE6-IE8 */
  url("../font/myfont.woff") format("woff"), /*! chrome, firefox */
  url("../font/myfont.ttf") format("truetype"), /*! chrome, firefox, opera, Safari, Android, iOS 4.2+ */
  url("../font/myfont.svg#myfont") format("svg"); /* iOS 4.1- */
  font-style: normal;
  font-weight: normal;
}
