.one{background:url(test/fixtures/one.png) right 10px top 10px no-repeat,linear-gradient(0deg,#efefef 0,#fff 100%)}
.two{background:url(test/fixtures/two.png) right 9px top 15px/9px 7px no-repeat,linear-gradient(-179deg,#fff 0,#f9f9f9 100%)}