.archive-header{margin-bottom:24px;margin-bottom:1.714285714rem}
.wp-caption.aligncenter,img.aligncenter{clear:both;margin-top:12px;margin-top:.857142857rem;margin-bottom:12px;margin-bottom:.857142857rem}
.wp-caption.aligncenter,.wp-caption.alignleft,.wp-caption.alignright{margin-bottom:24px;margin-bottom:1.714285714rem}
